/*---------------------------------------------------------------------------------
 * $Id: main.c 5 2007-02-14 06:37:05Z jos $
 * 
 * libhttpDS example code
---------------------------------------------------------------------------------*/

#include <nds.h>
#include <nds/arm9/console.h> //basic print funcionality
#include <stdlib.h>
#include <string.h>
#include <libhttp.h>

#include "wifi.h"

int main(void) {
  HTTP_REQUEST *request;
  HTTP_RESPONSE *response;
  unsigned char *buffer;
  unsigned long bufferSize;
  struct timeval tv;
  fd_set fdread;

  videoSetMode(0);  //not using the main screen
  videoSetModeSub(MODE_0_2D | DISPLAY_BG0_ACTIVE);  //sub bg 0 will be used to print text
  vramSetBankC(VRAM_C_SUB_BG);

  SUB_BG0_CR = BG_MAP_BASE(31);

  BG_PALETTE_SUB[255] = RGB15(31,31,31);  //by default font will be rendered with color 255

  //consoleInit() is a lot more flexible but this gets you up and running quick
  consoleInitDefault((u16*)SCREEN_BASE_BLOCK_SUB(31), (u16*)CHAR_BASE_BLOCK_SUB(0), 16);

  // Init WiFi hardware
  iprintf("Init WiFi hardware\n");
  initWifi();
 
  // Connecting via WFC data
  int wifiAssocStatus;
  iprintf("Connecting via WFC data\n");
  {
    Wifi_AutoConnect(); // request connect
    while(1) {
      wifiAssocStatus=Wifi_AssocStatus(); // check status
      if(wifiAssocStatus==ASSOCSTATUS_ASSOCIATED) {
        iprintf("Connected to AP\n");
        break;
      }
      if(wifiAssocStatus==ASSOCSTATUS_CANNOTCONNECT) {
        iprintf("Could not connect to AP\n");
        while(1);
        break;
      }
    }
  }
       
  // Create the request
  if (!(request = httpRequestNew()))
  {
    iprintf("could not allocate request.\n");
    return 0;
  }
  
  if (!(response = httpResponseNew()))
  {
    iprintf("could not allocate response.\n");
    return 0;
  }
  
  httpRequestInitialize(request, "GET", "/index.html", 1.0);
  httpRequestAttributeAdd(request, "Host", "ds.circuitdb.com", 0);
  httpRequestAttributeAdd(request, "User-Agent", "Nintendo DS", 0);

  buffer = httpRequestGetFullBuffer(request, &bufferSize);

  // Find the IP address of the server, with gethostbyname
  struct hostent * myhost = gethostbyname( "ds.circuitdb.com" );
  iprintf("Found IP Address\n");
 
  // Create a TCP socket
  int my_socket;
  my_socket = socket( AF_INET, SOCK_STREAM, 0 );
  iprintf("Created Socket\n");

  // Tell the socket to connect to the IP address we found, on port 80 (HTTP)
  struct sockaddr_in sain;
  sain.sin_family = AF_INET;
  sain.sin_port = htons(80);
  sain.sin_addr.s_addr= *( (unsigned long *)(myhost->h_addr_list[0]) );
  connect( my_socket,(struct sockaddr *)&sain, sizeof(sain) );
  iprintf("Connected to server\n");

  // send our request
  send(my_socket, buffer, strlen(buffer), 0 );
  iprintf("Sent request\n");

  // Print incoming data
  iprintf("Printing incoming data:\n");

  int do_loop = 1;  
  
  while (do_loop)
  {
  FD_ZERO(&fdread);
  FD_SET(my_socket, &fdread);

  tv.tv_sec  = 5;
  tv.tv_usec = 0;    

  unsigned char ret;
  int nread = 0;

  ioctl(my_socket, FIONREAD, &nread);
  
  ret = httpResponseRead(response, my_socket, nread);

    switch (ret)
    {
      case -1:
        iprintf("httpResponseRead() returned an error.\n");
        exit(0);
      case 0:
        iprintf("Read %d bytes.\n", nread);
        break;
      case 1:
        buffer = httpResponseGetFullBuffer(response, &bufferSize);
        iprintf("Read completed. Size = %lu\n", bufferSize);
        iprintf("%s\n", buffer);

        buffer = httpPacketGetPayload((HTTP_PACKET *) response, &bufferSize);
        //iprintf("Read completed. Size = %lu\n", bufferSize);
        //iprintf("%s\n", buffer);
        do_loop = 0;
    }      
  }

  iprintf("Other side closed connection!");

  shutdown(my_socket,0); // good practice to shutdown the socket.

  close(my_socket); // remove the socket.

  while(1);

  return 0;
}
