/*
 * $Id: libhttp.h 2 2006-10-24 18:55:58Z jos $
 *
 * libhttpDS
 *
 * Modifications by Jos Hendriks (jos@circuitdb.com)
 *
 * skape
 * mmiller@hick.org
 * 09/25/2002
 */
#ifndef _LIBHTTP_LIBHTTP_H
#define _LIBHTTP_LIBHTTP_H

/*
 * HttpAttribute.h
 */
typedef struct _http_attribute {

	char                   *name;
	char                   *value;

	struct _http_attribute *next;

} HTTP_ATTRIBUTE;

/*
 * HttpPacket.h
 */
#ifdef __cplusplus
extern "C" {
#endif

typedef struct _http_packet {

	HTTP_ATTRIBUTE *attributes;

	unsigned long  payloadLength;
	unsigned char  *payload;

	void           (*requestLineParser)(void *self, const char *requestBuffer);

	struct {
		
		unsigned char readingHeaders;

		unsigned char *header;
		unsigned long headerLength;

		unsigned long contentLength;
		unsigned long contentLengthLeft;

		unsigned char setPayloadOnClose;
	
	} internal;

} HTTP_PACKET;

void httpPacketInitialize(HTTP_PACKET *packet, void (*)(void *, const char *));

unsigned char httpPacketAttributeAdd(HTTP_PACKET *packet, const char *attributeName, const char *attributeValue, unsigned char overwriteIfExisting);
const char *httpPacketAttributeGet(HTTP_PACKET *packet, const char *attributeName);
HTTP_ATTRIBUTE *httpPacketAttributeEnum(HTTP_PACKET *packet, unsigned long index);
unsigned char httpPacketAttributeRemove(HTTP_PACKET *packet, const char *attributeName);
unsigned char httpPacketAttributesFlush(HTTP_PACKET *packet);

void httpPacketSetPayload(HTTP_PACKET *packet, unsigned char *payload, unsigned long payloadLength);
unsigned char *httpPacketGetPayload(HTTP_PACKET *packet, unsigned long *payloadLength);

unsigned char httpPacketRead(HTTP_PACKET *packet, int fd, unsigned long bytesToRead);
void httpPacketReadHeader(HTTP_PACKET *packet, unsigned char *header, const unsigned long headerLength);

void httpPacketEncodeString(const char *string, char *encodedString, unsigned long encodedStringLength);
void httpPacketExtractUriInformation(const char *uri, char *protocol, unsigned long protocolSize, char *hostname, unsigned long hostnameSize, unsigned long *port, char *path, unsigned long pathSize);

#ifdef __cplusplus
}
#endif

/*
 * HttpRequest.h
 */
#ifdef __cplusplus
extern "C" {
#endif

typedef struct _http_request {

	HTTP_PACKET		packet;

	char           *method;
	char           *uri;
	double         protocolVersion;

} HTTP_REQUEST;

HTTP_REQUEST *httpRequestNew();
void httpRequestDestroy(HTTP_REQUEST *request);
void httpRequestInitialize(HTTP_REQUEST *request, const char *method, const char *uri, double protocolVersion);

#define httpRequestAttributeAdd(request, attributeName, attributeValue, overwrite) httpPacketAttributeAdd((HTTP_PACKET *)request, attributeName, attributeValue, overwrite)
#define httpRequestAttributeGet(request, attributeName) httpPacketAttributeGet((HTTP_PACKET *)request, attributeName)
#define httpRequestAttributeEnum(request, index) httpPacketAttributeEnum((HTTP_PACKET *)request, index)
#define httpRequestAttributeRemove(request, attributeName) httpPacketAttributeRemove((HTTP_PACKET *)request, attributeName)
#define httpRequestAttributesFlush(request) httpPacketAttributesFlush((HTTP_PACKET *)request)
#define httpRequestSetPayload(request, payload, payloadLength) httpPacketSetPayload((HTTP_PACKET *)request, payload, payloadLength)

unsigned char *httpRequestGetFullBuffer(HTTP_REQUEST *request, unsigned long *requestBUfferLength);
void httpRequestFreeFullBuffer(HTTP_REQUEST *request, unsigned char *buffer);

#define httpRequestRead(request, fd, bytesToRead) httpPacketRead((HTTP_PACKET *)request, fd, bytesToRead)
unsigned char httpRequestWrite(HTTP_REQUEST *request, int fd);

/* Internal functions */
void _httpRequestParseRequestLine(void *request, const char *requestBuffer);

#ifdef __cplusplus
}
#endif

/*
 * HttpResponse.h
 */
typedef struct _http_response {

	HTTP_PACKET    packet;

	double         protocolVersion;
	unsigned long  responseCode;
	char           *responseString;

} HTTP_RESPONSE;

HTTP_RESPONSE *httpResponseNew();
void httpResponseDestroy(HTTP_RESPONSE *response);
void httpResponseInitialize(HTTP_RESPONSE *response, double protocolVersion, unsigned long responseCode, char *responseString);

#define httpResponseGetResponseCode(response) response->responseCode
#define httpResponseGetResponseString(response) response->responseString

#define httpResponseAttributeAdd(response, attributeName, attributeValue, overwrite) httpPacketAttributeAdd((HTTP_PACKET *)response, attributeName, attributeValue, overwrite)
#define httpResponseAttributeGet(response, attributeName) httpPacketAttributeGet((HTTP_PACKET *)response, attributeName)
#define httpResponseAttributeEnum(response, index) httpPacketAttributeEnum((HTTP_PACKET *)response, index)
#define httpResponseAttributeRemove(response, attributeName) httpPacketAttributeRemove((HTTP_PACKET *)response, attributeName)
#define httpResponseAttributesFlush(response) httpPacketAttributesFlush((HTTP_PACKET *)response)
#define httpResponseSetPayload(response, payload, payloadLength) httpPacketSetPayload((HTTP_PACKET *)response, payload, payloadLength)
#define httpResponseGetPayload(response, payloadLength) httpPacketGetPayload((HTTP_PACKET *)response, payloadLength)

unsigned char *httpResponseGetFullBuffer(HTTP_RESPONSE *response, unsigned long *responseBufferSize);
void httpResponseFreeFullBuffer(HTTP_RESPONSE *response, unsigned char *buffer);

#define httpResponseRead(request, fd, bytesToRead) httpPacketRead((HTTP_PACKET *)request, fd, bytesToRead)
void _httpResponseParseRequestLine(void *response, const char *requestBuffer);

#endif
